@@ -10,36 +10,20 @@ use Math::Prime::Util::Entropy;
 package Math::Prime::Util;
 use Carp qw/carp croak confess/;
 
-BEGIN {
-  use constant CSPRNG_CHACHA => 1;
-  use constant CSPRNG_ISAAC  => 0;
-}
-
 *_validate_num = \&Math::Prime::Util::PP::_validate_num;
 *_validate_integer = \&Math::Prime::Util::PP::_validate_integer;
 *_prime_memfreeall = \&Math::Prime::Util::PP::_prime_memfreeall;
 *prime_memfree  = \&Math::Prime::Util::PP::prime_memfree;
 *prime_precalc  = \&Math::Prime::Util::PP::prime_precalc;
 
-if (CSPRNG_CHACHA) {
-  require Math::Prime::Util::ChaCha;
-  *_is_csprng_well_seeded = \&Math::Prime::Util::ChaCha::_is_csprng_well_seeded;
-  *_csrand = \&Math::Prime::Util::ChaCha::csrand;
-  *_srand = \&Math::Prime::Util::ChaCha::srand;
-  *random_bytes = \&Math::Prime::Util::ChaCha::random_bytes;
-  *irand = \&Math::Prime::Util::ChaCha::irand;
-  *irand64 = \&Math::Prime::Util::ChaCha::irand64;
-} elsif (CSPRNG_ISAAC) {
-  require Math::Prime::Util::ISAAC;
-  *_is_csprng_well_seeded = \&Math::Prime::Util::ISAAC::_is_csprng_well_seeded;
-  *_csrand = \&Math::Prime::Util::ISAAC::csrand;
-  *_srand = \&Math::Prime::Util::ISAAC::srand;
-  *random_bytes = \&Math::Prime::Util::ISAAC::random_bytes;
-  *irand = \&Math::Prime::Util::ISAAC::irand;
-  *irand64 = \&Math::Prime::Util::ISAAC::irand64;
-} else {
-  die "Bad CSPRNG choice";
-}
+use Math::Prime::Util::ChaCha;
+*_is_csprng_well_seeded = \&Math::Prime::Util::ChaCha::_is_csprng_well_seeded;
+*_csrand = \&Math::Prime::Util::ChaCha::csrand;
+*_srand = \&Math::Prime::Util::ChaCha::srand;
+*random_bytes = \&Math::Prime::Util::ChaCha::random_bytes;
+*irand = \&Math::Prime::Util::ChaCha::irand;
+*irand64 = \&Math::Prime::Util::ChaCha::irand64;
+
 sub srand {
   my($seed) = @_;
   croak "secure option set, manual seeding disabled" if prime_get_config()->{'secure'};
@@ -121,6 +105,7 @@ sub entropy_bytes {
 *inverse_li = \&Math::Prime::Util::PP::inverse_li;
 *sieve_prime_cluster = \&Math::Prime::Util::PP::sieve_prime_cluster;
 *twin_prime_count = \&Math::Prime::Util::PP::twin_prime_count;
+*semiprime_count = \&Math::Prime::Util::PP::semiprime_count;
 *ramanujan_prime_count = \&Math::Prime::Util::PP::ramanujan_prime_count;
 *sum_primes = \&Math::Prime::Util::PP::sum_primes;
 *print_primes = \&Math::Prime::Util::PP::print_primes;
@@ -129,6 +114,9 @@ sub entropy_bytes {
 *is_quasi_carmichael = \&Math::Prime::Util::PP::is_quasi_carmichael;
 *is_pillai = \&Math::Prime::Util::PP::is_pillai;
 *is_fundamental = \&Math::Prime::Util::PP::is_fundamental;
+*is_semiprime = \&Math::Prime::Util::PP::is_semiprime;
+*is_totient = \&Math::Prime::Util::PP::is_totient;
+*is_square = \&Math::Prime::Util::PP::is_square;
 
 *random_prime = \&Math::Prime::Util::PP::random_prime;
 *random_ndigit_prime = \&Math::Prime::Util::PP::random_ndigit_prime;
@@ -140,37 +128,16 @@ sub entropy_bytes {
 *miller_rabin_random = \&Math::Prime::Util::PP::miller_rabin_random;
 *random_semiprime = \&Math::Prime::Util::PP::random_semiprime;
 *random_unrestricted_semiprime = \&Math::Prime::Util::PP::random_unrestricted_semiprime;
+*random_factored_integer = \&Math::Prime::Util::PP::random_factored_integer;
 
 *numtoperm = \&Math::Prime::Util::PP::numtoperm;
 *permtonum = \&Math::Prime::Util::PP::permtonum;
 *randperm = \&Math::Prime::Util::PP::randperm;
 *shuffle = \&Math::Prime::Util::PP::shuffle;
 
-sub moebius {
-  if (scalar @_ <= 1) {
-    my($n) = @_;
-    return 0 if defined $n && $n < 0;
-    _validate_num($n) || _validate_positive_integer($n);
-    return Math::Prime::Util::PP::moebius($n);
-  }
-  my($lo, $hi) = @_;
-  _validate_num($lo) || _validate_positive_integer($lo);
-  _validate_num($hi) || _validate_positive_integer($hi);
-  return Math::Prime::Util::PP::moebius_range($lo, $hi);
-}
+*moebius = \&Math::Prime::Util::PP::moebius;
+*euler_phi = \&Math::Prime::Util::PP::euler_phi;
 
-sub euler_phi {
-  if (scalar @_ <= 1) {
-    my($n) = @_;
-    return 0 if defined $n && $n < 0;
-    _validate_num($n) || _validate_positive_integer($n);
-    return Math::Prime::Util::PP::euler_phi($n);
-  }
-  my($lo, $hi) = @_;
-  _validate_num($lo) || _validate_positive_integer($lo);
-  _validate_num($hi) || _validate_positive_integer($hi);
-  return Math::Prime::Util::PP::euler_phi_range($lo, $hi);
-}
 sub jordan_totient {
   my($k, $n) = @_;
   _validate_positive_integer($k);
@@ -420,14 +387,9 @@ sub is_mersenne_prime {
 }
 sub is_square_free {
   my($n) = @_;
-  _validate_positive_integer($n);
+  _validate_integer($n);
   return Math::Prime::Util::PP::is_square_free($n);
 }
-sub is_semiprime {
-  my($n) = @_;
-  _validate_positive_integer($n);
-  return Math::Prime::Util::PP::is_semiprime($n);
-}
 sub is_primitive_root {
   my($a,$n) = @_;
   return 0 if $n == 0;
@@ -734,11 +696,6 @@ sub is_power {
   $vn = '-'.$vn if $n < 0;
   return Math::Prime::Util::PP::is_power($vn, $a, $refp);
 }
-sub is_square {
-  my($n) = @_;
-  return 0 if defined $n && int($n) < 0;
-  return Math::Prime::Util::PP::is_power($n,2);
-}
 sub is_prime_power {
   my($n, $refp) = @_;
   my $vn = "$n";  $vn =~ s/^-//;
@@ -803,48 +760,20 @@ sub forprimes (&$;$) {    ## no critic qw(ProhibitSubroutinePrototypes)
 }
 
 sub forcomposites(&$;$) { ## no critic qw(ProhibitSubroutinePrototypes)
-  my($sub, $beg, $end) = @_;
-  if (!defined $end) { $end = $beg; $beg = 4; }
-  _validate_num($beg) || _validate_positive_integer($beg);
-  _validate_num($end) || _validate_positive_integer($end);
-  $beg = 4 if $beg < 4;
-  $end = Math::BigInt->new(''.~0) if ref($end) ne 'Math::BigInt' && $end == ~0;
-  my $oldforexit = _start_for_loop();
-  {
-    my $pp;
-    local *_ = \$pp;
-    for ( ; $beg <= $end ; $beg++ ) {
-      if (!is_prime($beg)) {
-        $pp = $beg;
-        $sub->();
-      last if $_exitloop;
-      }
-    }
-  }
-  _end_for_loop($oldforexit);
+  Math::Prime::Util::_generic_forcomp_sub('composites', @_);
+}
+sub foroddcomposites(&$;$) { ## no critic qw(ProhibitSubroutinePrototypes)
+  Math::Prime::Util::_generic_forcomp_sub('oddcomposites', @_);
+}
+sub forsemiprimes(&$;$) { ## no critic qw(ProhibitSubroutinePrototypes)
+  Math::Prime::Util::_generic_forcomp_sub('semiprimes', @_);
 }
 
-sub foroddcomposites(&$;$) { ## no critic qw(ProhibitSubroutinePrototypes)
-  my($sub, $beg, $end) = @_;
-  if (!defined $end) { $end = $beg; $beg = 9; }
-  _validate_num($beg) || _validate_positive_integer($beg);
-  _validate_num($end) || _validate_positive_integer($end);
-  $beg = 9 if $beg < 9;
-  $beg++ unless $beg & 1;
-  $end = Math::BigInt->new(''.~0) if ref($end) ne 'Math::BigInt' && $end == ~0;
-  my $oldforexit = _start_for_loop();
-  {
-    my $pp;
-    local *_ = \$pp;
-    for ( ; $beg <= $end ; $beg += 2 ) {
-      if (!is_prime($beg)) {
-        $pp = $beg;
-        $sub->();
-        last if $_exitloop;
-      }
-    }
-  }
-  _end_for_loop($oldforexit);
+sub forfactored(&$;$) { ## no critic qw(ProhibitSubroutinePrototypes)
+  Math::Prime::Util::_generic_forfac(0, @_);
+}
+sub forsquarefree(&$;$) { ## no critic qw(ProhibitSubroutinePrototypes)
+  Math::Prime::Util::_generic_forfac(1, @_);
 }
 
 sub fordivisors (&$) {    ## no critic qw(ProhibitSubroutinePrototypes)
